
from Additional_functions import *
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeBar, GaugeTypeValue
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin


class Gauge1MinGaugeBase(GaugePowerAnalyzerMixin,GaugeTypeBar):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.publishing_sensors = 'PF'
        self.trendExistence = True
        self.gauge_color = ['blue'] *3
        self.success = True

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p


class GaugeVRMS(Gauge1MinGaugeBase):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'VRMS'
        self.gauge_name = 'VRMS'
        self.rank = 31
        self.trends = {"trendYMax" : 1000, "trendYMin" : -1}
        self.units = 'volt'

    def compute(self):
        # self.mart = {'Phase 1': self.voltages[0], 'Phase 2': self.voltages[1], 'Phase 3': self.voltages[2], 'trendYMax': 1000, 'trendYMin': -1, 'Message': -999}
        if len(self.voltages) < 3:
            self.gauge_color = ['black','black','black']
            self.success = False
            self.kpis = {'Phase 1': -999, 'Phase 2': -999, 'Phase 3': -999}
            return
        
        self.kpis = {'Phase 1': self.voltages[0], 'Phase 2': self.voltages[1], 'Phase 3': self.voltages[2]}
        self.mart_extras = {'Message': -999}


class GaugeIRMS(Gauge1MinGaugeBase):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'IRMS'
        self.gauge_name = 'IRMS'
        self.rank = 32
        self.trends = {"trendYMax" : 1000, "trendYMin" : -1}
        self.units = 'amp'

    def compute(self):
        # self.mart = {'Phase 1': self.currents[0], 'Phase 2': self.currents[1], 'Phase 3': self.currents[2], 'trendYMax': 1000, 'trendYMin': -1, 'Message': -999}
        if len(self.currents) < 3:
            raise GaugeMissingData("No currents")

            # self.gauge_color = ['black','black','black']
            # self.success = False
            # self.kpis = {'Phase 1': -999, 'Phase 2': -999, 'Phase 3': -999}
            # self.values = [-999,-999,-999]
            # return

        self.kpis = {'Phase 1': self.currents[0], 'Phase 2': self.currents[1], 'Phase 3': self.currents[2]}
        self.mart_extras = {'Message': -999}

class GaugePowerFactor(Gauge1MinGaugeBase):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Power Factor'
        self.gauge_name = 'Power Factor'
        self.rank = 33
        self.trends = {"trendYMax" : 1, "trendYMin" : -1}
        self.units = 'values'


    def compute(self):
        # self.mart = {'Phase 1': self.pfs[0], 'Phase 2': self.pfs[1], 'Phase 3': self.pfs[2], 'trendYMax': 1, 'trendYMin': -1, 'Message': -999}
        if len(self.pfs) < 3:
            raise GaugeMissingData("No pfs")
            # self.gauge_color = ['black','black','black']
            # self.success = False
            # self.kpis = {'Phase 1': -999, 'Phase 2': -999, 'Phase 3': -999}
            # self.values = [-999,-999,-999]
            # return

        self.kpis = {'Phase 1': self.pfs[0], 'Phase 2': self.pfs[1], 'Phase 3': self.pfs[2]}
        self.mart_extras = {'Message': -999}

class GaugeActivePower(Gauge1MinGaugeBase):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Active Power'
        self.gauge_name = 'Active Power'
        self.rank = 34
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdoutput' else True
        self.trends = {"trendYMax" : 4000, "trendYMin" : -1}
        self.units = 'kilowatt'


    def compute(self):
        # self.mart = {'Phase 1': self.powers[0], 'Phase 2': self.powers[0], 'Phase 3': self.powers[0], 'trendYMax': 4000, 'trendYMin': -1, 'Message': -999}
        if len(self.powers) < 3:
            raise GaugeMissingData("No powers")
            # self.success = False
            # self.gauge_color = ['black','black','black']
            # self.kpis = {'Phase 1': -999, 'Phase 2': -999, 'Phase 3': -999}
            # self.values = [-999,-999,-999]
            # return

        self.kpis = {'Phase 1': self.powers[0], 'Phase 2': self.powers[1], 'Phase 3': self.powers[2]}
        self.mart_extras = {'Message': -999}


class GaugeActiveEnergyTotal(GaugePowerAnalyzerMixin,GaugeTypeValue):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Active Energy Total'
        self.gauge_name = 'Active Energy Total'
        self.rank = 34
        self.publishing_sensors = 'PF'
        self.trendExistence = True
        self.gauge_color = ['green']
        self.success = True

        self.hideFromGroup = False if self.subassembly.lower() != 'vfdoutput' else True
        self.trends = {"trendYMax" : 4000, "trendYMin" : -1}
        self.units = 'kilowatt'


    def compute(self):
        self.kpis = {'trend': self.active_energy_total}
        self.mart_extras = {'Message': -999}
