from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *
from powercompute_utils import compute_current_imbalance
import numpy as np

class GaugeCurrentImbalance(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__="current_imbalance"
    __pm_function_name__="Current Imbalance"

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]

    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name']  = '__test_current_imbalance'
        super().__init__(*args,**kwargs)
        self.gauge_name = 'Current Imbalance'
        self.gauge_output_name = 'Current Imbalance'
        self.publishing_sensors = "PF"
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.trendExistence = True
        self.gaugeLife = 600
        self.rank = 11
        self.feedback = True
        self.mro = True
        self.userInput = True
        self.hasHourglass = True
        self.hasAlarm = True
        self.trends = {"trendYMax" : 300, "trendYMin" : 0}

    def compute_current_imbalance(self,i_3phase):
        return compute_current_imbalance(i_3phase)
    
    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p

    def compute(self):
        activate_hour_glass = False
        '''
            CURRENT IMBALANCE____________
        '''

        self.i_avg = np.mean(self.i_3phase)
        print("\n\n i_3phase is {} with value  {}  &&  i_avg {} computed \n\n".format(type(self.i_3phase),self.i_3phase,self.i_avg))

        x = self.compute_current_imbalance(self.i_3phase)
        current_deviation = x['current_deviation']
        self.i_imbalance = x['i_imbalance']

        yellow_threshold = self.get_gauge_rule_value("yellow_threshold",max( 0.1 * self.i_avg, 0.02 * self.cttype * self.k_factor ))
        red_threshold = self.get_gauge_rule_value("red_threshold",max( 0.15 * self.i_avg, 0.05 * self.cttype * self.k_factor )) 

        current_imbalance_color      = 'red' if self.i_imbalance >= red_threshold else 'yellow' if  self.i_imbalance >= yellow_threshold else 'green'
        current_imbalance_indicator  = list(map(lambda x: 'red' if x >= red_threshold else 'yellow' if x >= yellow_threshold else 'green', current_deviation))
        self.update_indicators(current_imbalance_indicator)
        self.mart_extras = {'red_threshold': red_threshold,'yellow_threshold': yellow_threshold}
        if self.Machine_status != 0:
            # Machine is running means machine success is true.
            # The color will be published. The last known success time and color will be updated.
            self.gauge_color        = current_imbalance_color
            last_known_success = {'indicator': self.Machine_status,  'time': self.current_datetime_stamp, 'color': self.gauge_color }
            self.reason = self.get_text_under_gauge()
            self.success = True
            self.kpis = {'Phase 1':current_deviation[0] , 'Phase 2':current_deviation[1] , 'Phase 3':current_deviation[2]}
            self.values = [-1]
        else:
            # else condition is for success is False. i.e. self.Machine_status is 0
            # For sucess is false the gauge_color will be the last known success color within 24 hours else gauge color will be black
            print("Else...")

            self.gauge_color=["black"]
            self.success = False
            self.reason = "Machine Idle"
            self.kpis = {'Phase 1':-999 , 'Phase 2':-999 ,  'Phase 3':-999}
            self.values = [-1]
        



