
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeBar
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin

class GaugeCurrentThd(GaugePowerAnalyzerMixin,GaugeTypeBar):

    __internal_gauge_name__= 'current_thd'
    __pm_function_name__ = 'Current Harmonic Distortion'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]

    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_current_thd'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Current Harmonic Distortion'
        self.gauge_name = 'Current Harmonic Distortion'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 42
        self.trendExistence = True
        self.gaugeLife = 600
        self.userInput = True
        self.gauge_color = ['white'] * 3
        self.hasAlarm = False   # Looks like it doesn't use alarm, so this isn't needed.
        self.trends = {"trendYMax" : 100, "trendYMin" : 0}
        self.units = "%"
        self.kpis = {'Phase 1': None, 'Phase 2': None, 'Phase 3': None}

    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p

    def compute(self):
        '''
        CURRENT THD__________________
        '''
        yellow_threshold = self.get_gauge_rule_value("yellow_threshold",10)
        red_threshold = self.get_gauge_rule_value("red_threshold",20) 
        self.gauge_color           = list(map(lambda x: 'red' if x >= red_threshold else 'yellow' if  x >= yellow_threshold else 'green', self.ithd))
        self.kpis = {'Phase 1': self.ithd[0], 'Phase 2': self.ithd[1], 'Phase 3': self.ithd[2]}
        self.success = True


