
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *
import numpy as np

class GaugeFrequencyVariation(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'frequency_variation'
    __pm_function_name__ = 'Frequency Variation'

    __rules__ = [
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]


    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_frequency_variation'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Frequency Variation'
        self.gauge_name = 'Frequency Variation'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 17
        self.gaugeLife = 600
        self.trendExistence = True
        self.mro = True
        self.feedback = True
        self.userInput = True
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdoutput' else True
        self.hasHourglass = True
        self.hasAlarm = True
        self.feedback_colors = ["green","red"] 
        self.trends = {"trendYMax" : 10, "trendYMin" : 0}

    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p


    def compute(self):
        current_timestamp = self.stored_object.current_ts()
        # gauge_name  = 'Frequency Variation'
            
        #effective_fund_freq = 50 if int(min_10_list[24]) in range(45, 55) else 60
        effective_fund_freq = 50 if int(self.fund_freq) in range(45, 55) else 60
            
        if len(self.freq_var_val)==0:
            freq_var=0
        else:
            freq_var = (np.mean((self.freq_var_val - effective_fund_freq)**2))**0.5
                
        red_threshold_freq = self.get_gauge_rule_value("red_threshold",1.5) 

        # red_threshold_freq = 1.5
            
        # input_red_threshold  = get_threshold(self.API_list,self.subassembly_instance, 'Frequency Variation', "red_threshold"  )
        # red_threshold_freq        = 1.5 if input_red_threshold == None else input_red_threshold        
            
        freq_color    = 'red' if freq_var > red_threshold_freq else 'green'
        
        freq_variation_indicator  = list(map(lambda x: 'red' if x >= red_threshold_freq else 'green', [freq_var]))
            
        red_indicator    = red_yellow_event_indicator('red', freq_variation_indicator)
        yellow_indicator = [0] # red_yellow_event_indicator('yellow', current_imbalance_indicator)
        # event_indicator  = min(red_indicator[0] + yellow_indicator[0], 1)
            
        # self.stored_object.current()[self.subassembly_instance].update({gauge_name:{'events':event_indicator, 'y_indicator':yellow_indicator, 'r_indicator':red_indicator }})
        self.mart_extras = {'red_threshold': red_threshold_freq}
        if self.Machine_status != 0:
            #freq_var      = (np.mean((min_10_list[25] - effective_fund_freq)**2))**0.5
            #freq_color    = 'red' if freq_var > red_threshold_freq else 'green'

            self.gauge_color        = freq_color
            # last_known_success = {'indicator': self.Machine_status,  'time': current_timestamp, 'color': self.gauge_color }
            # self.stored_object.current()[self.subassembly_instance].update({gauge_name:{'last_known_success': last_known_success}})

            # #print('\n before hourglass function')
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, gauge_name, self.Machine_status, [freq_var])
            # #print('\n after hourglass function: ', hourglass_color_value_reason_call)
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]
            # freq_var   			= hourglass_color_value_reason_call[2]  # value
            # reason              = hourglass_color_value_reason_call[3]
                
            #reason              =reason if reason != '' else text_under_gauge
            #print('\n activate hour glass', activate_hour_glass)
            self.success = True
            print("Freq_var is: ",freq_var)
            # self.mart = {'trend':freq_var , 'red_threshold': red_threshold_freq, 'trendYMax': 10 , 'trendYMin': 0}
            self.kpis = {'trend':freq_var}
            

            # freq_output  = {'color': [gauge_color], 'mart': {'trend':freq_var[0] , 'red_threshold': red_threshold_freq, 'trendYMax': 10 , 'trendYMin': 0},#, 'Message': -999},                                  
            #                 'success': 'true', 'rank': 17, 'trendExistence': True, 
            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True,  
            #                 'mro': True, 
            #                 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'red'],  
            #                 'reason' : reason, 'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
            
        else:
            self.reason = "Machine Idle"
            self.gauge_color = ["black"]
            # self.mart = {'trend':-999 , 'red_threshold': red_threshold_freq, 'trendYMax': 10 , 'trendYMin': 0}
            self.kpis = {'trend':-999}
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, gauge_name, self.Machine_status, [])
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # #gauge_color         = hourglass_color_value_reason_call[1]

            # freq_output  = {'color': ['black'],'mart': {'trend':-999 , 'red_threshold': red_threshold_freq, 'trendYMax': 10 , 'trendYMin': 0},# 'Message': "Machine Idle"},                                  
            #                 'success': 'false', 'rank': 17, 'trendExistence': True, 
            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True,  
            #                 'mro': True, 
            #                 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'red'],  
            #                                 'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
        
        # self.update_historical_ts()

        # self.add_gauge_alarm(freq_output,gauge_name,current_timestamp,self.machineid,self.subassembly_instance)
        
        # print('\n_____________after freq var  ________________\n')
        # return freq_output

