
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeBar
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *


class GaugeHeaterFailure(GaugePowerAnalyzerMixin,GaugeTypeBar):
    __internal_gauge_name__= 'heater_failure'
    __pm_function_name__ = 'Heater Failure'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]

    def __init__(self,*args,**kwargs):
        print("*************** HEATER FAILURE ****************************")
        kwargs['internal_gauge_name'] = '__test_heater_failure'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Heater Failure'
        self.gauge_name = 'Heater Failure'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.gauge_color = ['white']*3
        self.rank = 43
        self.gaugeLife = 1800
        self.trendExistence = True
        self.feedback = True
        self.userInput = True
        self.mro = True
        self.hasAlarm=True
        self.reliability_quotient_output = 100
        self.trends = {"trendYMax" : 3, "trendYMin" : 0}
        self.alarm_names = [ 'Phase 1', 'Phase 2', 'Phase 3']
        self.units = "values"
        self.kpis = {'Phase 1':None, 'Phase 2':None ,  'Phase 3':None}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        p["gaugePropertiesOutput"] = { "reliabilityQuotient" : self.reliability_quotient_output }
        return p


    def compute(self):
        ab1 = self.av[0]
        ab2 = self.av[1]
        ab3 = self.av[2]
        gr_limit = 0.05  
        yr_limit = 0.2

        if ab1 < gr_limit and ab2 < gr_limit and ab3< gr_limit:
            self.values = [0,0,0]
            self.gauge_color = ['green']*3
        elif ab1<gr_limit and ab2>gr_limit and ab2<yr_limit and ab3>gr_limit and ab3<yr_limit:
            self.values = [0,0,1]
            self.gauge_color = ['green', 'green', 'yellow']
        elif ab1>gr_limit and ab1<yr_limit and ab2<gr_limit and ab3<yr_limit:
            self.values = [1,0,0]
            self.gauge_color = ['yellow', 'green', 'green']
        elif ab1<yr_limit and ab2>gr_limit and ab2<yr_limit and ab3<gr_limit:
            self.values = [0,1,0]
            self.gauge_color = ['green', 'yellow', 'green']
        elif ab1<gr_limit and ab2>yr_limit and ab3>yr_limit:
            self.values = [0,0,2]
            self.gauge_color = ['green', 'green', 'red']
        elif ab1>yr_limit and ab2<gr_limit and ab3>yr_limit:
            self.values = [2,0,0]
            self.gauge_color = ['red', 'green', 'green']
        elif ab1>yr_limit and ab2>yr_limit and ab3<gr_limit:
            self.values = [0,2,0]
            self.gauge_color = ['green', 'red', 'yellow']
        else:
            self.values = [0]*3
            self.gauge_color = ['green']*3

        yellow_threshold = self.get_gauge_rule_value("yellow_threshold",1)
        red_threshold = self.get_gauge_rule_value("red_threshold",2) 


        if self.Machine_status != 0:

            self.kpis = {'Phase 1':self.values[0], 'Phase 2':self.values[1] ,  'Phase 3':self.values[2]}
            self.mart_extras = {"red_threshold": red_threshold, "yellow_threshold": yellow_threshold}
            self.success = True
        else:
            self.gauge_color=["black"]*3
            self.reason = "Machine Idle"
            self.reliability_quotient_output = -1
            self.kpis = {'Phase 1':-999, 'Phase 2':-999 ,  'Phase 3':-999}
            self.mart_extras = {"red_threshold": red_threshold, "yellow_threshold": yellow_threshold}
