
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin


class GaugeMotorBearingFailure(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'motor_bearing_failure'
    __pm_function_name__ = 'Motor Bearing Failure'

    __rules__ = [
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]


    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_motor_bearing_failure'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Motor Bearing Failure'
        self.gauge_name = 'Motor Bearing Failure'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.reliability_quotient_output = 100
        self.rank = 19
        self.gaugeLife = 3600
        self.trendExistence = True
        self.mro = True
        self.feedback = True
        self.userInput = True
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdinput' else True
        self.hasHourglass = True
        self.hasAlarm = True
        self.feedback_colors = ["green","red"]
        self.trends = {"trendYMax" : 30, "trendYMin" : 0}
        self.units = "values"

    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        p["gaugePropertiesOutput"] = { "reliabilityQuotient" : self.reliability_quotient_output }
        return p


    def compute(self):
        self.reliability_quotient_output = 100
        pf_median     = self.pf_median_list[1]
        trend_value   = self.pf_median_list[0] if pf_median >= 0.4 else -999
        bearing_failure_index =  0.65 * trend_value
        red_threshold = self.get_gauge_rule_value("red_threshold",15) 

        bearing_failure_color = 'red' if bearing_failure_index >= red_threshold else 'green'

        self.mart_extras = {"red_threshold": red_threshold}  
        if self.Machine_status != 0:
            self.gauge_color        = bearing_failure_color
            self.success = True
            self.kpis = { "trend" : trend_value }
        else:
            self.gauge_color = ['black']
            self.reason = "Machine Idle"
            self.reliabilityQuotent = -1
            self.kpis = { "trend" : -999 }



