
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *
from itertools import groupby

class GaugeOverVoltage(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'over_voltage'
    __pm_function_name__ = 'Over Voltage'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        }
    ]


    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_over_voltage'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Over Voltage'
        self.gauge_name = 'Over Voltage'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.gaugeLife = 600
        self.trendExistence = True
        self.feedback = True
        self.userInput = True
        self.rank = 16
        self.mro = True
        self.hasHourglass = True
        self.hasAlarm = True
        self.feedbackColors = ["green","yellow"]
        self.trends = {"trendYMax" : 10, "trendYMin" : 0}
        self.units="count"
        
    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']},"events": {"yellow":{},"red":{}},'batch_v_min':{}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p

    def get_worst_event_info(self):
        return self.get_worst_event_data_max()

    def get_worst_event_message(self):
        worst_event_info = self.get_worst_event_info()
        last_worst_event_time = worst_event_info['ts']
        last_event_time_format = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(int(last_worst_event_time)) )
        worst_event_message  = ';  Worst Event: Phases' + event_phases(worst_event_info['event'][-1]) + ',  '+ self.gauge_name +' value: ' + str(worst_event_info['event'][0]) + ',  Time: ' + str(last_event_time_format) + ' UTC'
        return worst_event_message

    def compute(self):
        current_timestamp = self.stored_object.current_ts() 
        
        print('\n batch v max: ', self.batch_voltage_max)
        
        ov_run_length   = []   
        
        for each_phase in self.voltage_overrun:    
            rle = [(k, sum(1 for i in g)) for k, g in groupby(each_phase)]      # using itertools: from itertools import groupby
            j = 0     
            for i in rle:
                if i[1]>60 and i[0]==1:
                    j+=1
            ov_run_length = ov_run_length + [j]
        
        ov_trend    = ov_run_length
        #print('\n ov trend: ', ov_trend)
        
        yellow_threshold = self.get_gauge_rule_value("yellow_threshold",1)

        # input_yellow_threshold  = get_threshold(self.API_list,self.subassembly_instance, 'over voltage', "yellow_threshold"  )
        # yellow_threshold        = 1 if input_yellow_threshold == None else input_yellow_threshold
        
        self.overvoltage_color      = 'green' if max(ov_trend) < yellow_threshold else 'yellow'
        #print('\n over v color: ', overvoltage_color)
            
        ov_color_indicator = list(map(lambda x: 'yellow' if x>= yellow_threshold else 'green', ov_trend))
        #print('\n ov color indicator:', ov_color_indicator)
            
        # yellow_indicator = red_yellow_event_indicator('yellow', ov_color_indicator)
        # event_indicator  = yellow_indicator[0]
        # #print('\n event indicator: ', event_indicator)
        # #print('\n before ov indicator, batch v max: ', batch_voltage_max )
        batch_val= max(self.batch_voltage_max)
        ov_indicator  = calc_indicator(self.batch_voltage_max,batch_val)
            
        # self.stored_object.current()[self.subassembly_instance].update({self.gauge_name:{'events':event_indicator, 'y_indicator':yellow_indicator, 'batch_v_min': ov_indicator, 'Ph_1_run_value':ov_trend[0], 'Ph_2_run_value':ov_trend[1], 'Ph_3_run_value':ov_trend[2], 'v1': self.batch_voltage_max[0], 'v2': self.batch_voltage_max[1], 'v3': self.batch_voltage_max[2] }})
        #print('\n stored obj: \n', self.stored_object.current()[subassembly_instance][self.gauge_name])
        # self.update_indicators(ov_color_indicator,extras={'batch_v_min': ov_indicator, 'Ph_1_run_value':ov_trend[0], 'Ph_2_run_value':ov_trend[1], 'Ph_3_run_value':ov_trend[2], 'v1': self.batch_voltage_max[0], 'v2': self.batch_voltage_max[1], 'v3': self.batch_voltage_max[2]})
        self.update_indicators(ov_color_indicator,extras={'batch_v_min': ov_indicator})
        self.mart_extras = {'yellow_threshold': yellow_threshold}
        if self.Machine_status != 0:
            self.gauge_color        = self.overvoltage_color
            # last_known_success = {'indicator': self.Machine_status,  'time': current_timestamp, 'color': self.gauge_color }
            # self.stored_object.current()[self.subassembly_instance][self.gauge_name].update({'last_known_success': last_known_success})
            
            # self.reason = self.stored_object.worst_event_text(self.gauge_name, self.subassembly_instance)
            self.reason = self.get_worst_event_text()

            #print('\n text under gauge: ', text_under_gauge)

            #print('\n before hourglass function')
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, self.gauge_name, self.Machine_status, ov_trend)
            # #print('\n after hourglass function: ', hourglass_color_value_reason_call)
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]
            # ov_trend   = hourglass_color_value_reason_call[2]  # value
            # reason              = hourglass_color_value_reason_call[3]
                
            # reason              =reason if reason != '' else text_under_gauge

            self.success = True
            # self.mart = {'Phase 1':ov_trend[0] , 'Phase 2':ov_trend[1] ,  'Phase 3':ov_trend[2] , 'yellow_threshold': yellow_threshold, 'trendYMax': 10 , 'trendYMin': 0}
            self.kpis = {'Phase 1':ov_trend[0] , 'Phase 2':ov_trend[1] ,  'Phase 3':ov_trend[2]}
            #print('\n activate hour glass', activate_hour_glass)

            # overvoltage_output  = { 'color': [gauge_color], 'mart': {'Phase 1':ov_trend[0] , 'Phase 2':ov_trend[1] ,  'Phase 3':ov_trend[2] , 'yellow_threshold': yellow_threshold, 'trendYMax': 10 , 'trendYMin': 0},# 'Message': -999},                                  
            #                         'success': 'true',  "reason": text_under_gauge, 'rank': 16, 'trendExistence': True, 
            #                         'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True, 
            #                         'mro': True, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow'],  
            #                         'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
            
        else:
            self.gauge_color = ['black']
            self.reason = 'Machine Idle'
            # self.mart = {'Phase 1':-999 , 'Phase 2': -999 ,  'Phase 3':-999 , 'yellow_threshold': yellow_threshold, 'trendYMax': 10 , 'trendYMin': 0}
            self.kpis = {'Phase 1':-999 , 'Phase 2': -999 ,  'Phase 3':-999 }
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, self.gauge_name, self.Machine_status, [])
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]


            # overvoltage_output  = { 'color': ['black'],'mart': {'Phase 1':-999 , 'Phase 2': -999 ,  'Phase 3':-999 , 'yellow_threshold': yellow_threshold, 'trendYMax': 10 , 'trendYMin': 0},# 'Message': 'Machine Idle'},                                  
            #                         'success': 'false',  "reason": "Machine Idle", 'rank': 16, 'trendExistence': True, 
            #                         'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True,  
            #                         'mro': True, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow'],  
            #                         'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}


        # self.update_historical_ts()

        # self.add_gauge_alarm(overvoltage_output,self.gauge_name,current_timestamp,self.machineid,self.subassembly_instance)
            
        # print('\n_____________after ov  ________________\n')
        # return overvoltage_output

