
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeValue
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *
import numpy as np

class GaugePowerUtilizationUnderLoad(GaugePowerAnalyzerMixin,GaugeTypeValue):
    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_utilization_under_load'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Utilization Under Load'
        self.gauge_name = 'Utilization Under Load'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 2
        self.gaugeLife = 3600
        self.trendExistence = True
        self.userInput = True
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdinput' else True
        self.trends = {"trendYMax" : 100, "trendYMin" : 0}
        self.units = "%"

    def get_default_stored_object(self):
            return {'historical_ts':{}, 'last_known_success':{'true':0,'false':0, 'color':['white']}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        p['unit'] = self.units
        return p


    def compute(self):
        self.mystore()["historical_ts"].update({str(self.current_datetime_stamp): self.percent_machine_running})
        batches_available = len(self.mystore()['historical_ts'])
        self.reason = 'data available for: ' + str(int(batches_available/6)) + ' hours only' if batches_available < 24*6 else ''
        machine_running_percent = np.mean(np.array(list(self.mystore()['historical_ts'].values())))
        self.purge_stored_ts_by_time(self.mystore()['historical_ts'],24*60*60)
        
        if np.isnan(machine_running_percent):
             machine_running_percent = 0
        self.success = True
        # self.mart = { "trend" : machine_running_percent, "trendYMax" : 100, "trendYMin" : 0 }
        self.kpis = { "trend" : machine_running_percent }
        self.gauge_color = ["green"]
        self.values = [machine_running_percent]

