
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *


class GaugeSag(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'sag'
    __pm_function_name__ = 'Sag'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }

    ]

    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_sag'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Sag'
        self.gauge_name = 'Sag'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 13
        self.gaugeLife = 600
        self.trendExistence = True
        self.mro = True
        self.feedback = True
        self.userInput = True
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdoutput' else True
        self.hasHourglass = True
        self.hasAlarm = True
        self.trends = {"trendYMax" : 500, "trendYMin" : 0}
        self.units = "count"
        
    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']},"events": {"yellow":{},"red":{}},'batch_v_min':{}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p

    def get_worst_event_info(self):
        return self.get_worst_event_data_min()

    def get_worst_event_message(self):
        worst_event_info = self.get_worst_event_info()
        p_u_value = round(100*(1-worst_event_info['event'][0]),3)
        last_worst_event_time = worst_event_info['ts']
        last_event_time_format = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(int(last_worst_event_time)) )
        worst_event_message  = ';  Worst Event: Phases' + event_phases(worst_event_info['event'][-1]) + ', P.U.(%): ' + str(p_u_value) + ',  Time: ' + str(last_event_time_format) + ' UTC'
        return worst_event_message


    def compute(self):
        current_timestamp = self.stored_object.current_ts()
        max_sag_count   = max(self.sag_count)
            
        #sag_swell_mag_min = min_10_list[31]
        #print('\n sag swell mag: ',sag_swell_mag_min )
        
        yellow_threshold_sag = self.get_gauge_rule_value("yellow_threshold",1)
        red_threshold_sag = self.get_gauge_rule_value("red_threshold",3) 

        # input_yellow_threshold_sag       = get_threshold(self.API_list,self.subassembly_instance, 'sag', "yellow_threshold"  )
        # input_red_threshold_sag          = get_threshold(self.API_list,self.subassembly_instance, 'sag', "red_threshold"  )
        
        # yellow_threshold_sag             = 1 if input_yellow_threshold_sag == None else input_yellow_threshold_sag
        # red_threshold_sag                = 3 if input_red_threshold_sag == None else input_red_threshold_sag
        
        self.gauge_color   = 'red' if max_sag_count >= red_threshold_sag else 'yellow' if  max_sag_count >= yellow_threshold_sag else 'green'
            
        sag_color_indicator = list(map(lambda x: 'red' if x >= red_threshold_sag else 'yellow' if x >= yellow_threshold_sag else 'green', self.sag_count))
        # red_indicator    = red_yellow_event_indicator('red', sag_color_indicator)
        # yellow_indicator = red_yellow_event_indicator('yellow', sag_color_indicator)
        # event_indicator  = min(red_indicator[0] + yellow_indicator[0], 1)
        #print('\n sag color indicator: ', sag_color_indicator)
        #print('\n red indicator: ', red_indicator)
        #print('\n yellow indicator: ', yellow_indicator)
        #print('\n event indicator: ', event_indicator)
            
        batch_min = min(self.sag_swell_mag_min)
        self.sag_indicator  = calc_indicator(self.sag_swell_mag_min,batch_min)

        # self.stored_object.current()[self.subassembly_instance].update({self.gauge_name:{'events':event_indicator, 'y_indicator':yellow_indicator,'r_indicator':red_indicator, 'batch_v_min': self.sag_indicator }})
        self.update_indicators(sag_color_indicator,extras={'batch_v_min': self.sag_indicator })
        #print('\n stored_obj: ', self.stored_object.current()[subassembly_instance] )
        self.mart_extras = {'red_threshold': red_threshold_sag, 'yellow_threshold': yellow_threshold_sag}
        if self. Machine_status != 0:
            # last_known_success = {'indicator': self.Machine_status,  'time': current_timestamp, 'color': self.gauge_color }
            # self.stored_object.current()[self.subassembly_instance][self.gauge_name].update({'last_known_success': last_known_success})

            # self.reason = self.stored_object.worst_event_text(self.gauge_name, self.subassembly_instance)
            self.reason = self.get_worst_event_text()
            #print('\n text under gauge: ', text_under_gauge)

            #print('\n before hourglass function')
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, gauge_name, self.Machine_status, self.sag_count)
            # #print('\n after hourglass function: ', hourglass_color_value_reason_call)
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]
            # sag_count   = hourglass_color_value_reason_call[2]  # value
            # reason              = hourglass_color_value_reason_call[3]
                
            # reason              =reason if reason != '' else text_under_gauge

            self.success = True
            # self.mart = {'Phase 1':self.sag_count[0] , 'Phase 2':self.sag_count[1] ,  'Phase 3':self.sag_count[2] , 
            #                 'red_threshold': red_threshold_sag, 'yellow_threshold': yellow_threshold_sag, 'trendYMax': 500 , 'trendYMin': 0}
            self.kpis = {'Phase 1':self.sag_count[0] , 'Phase 2':self.sag_count[1] ,  'Phase 3':self.sag_count[2]}
            #print('\n activate hour glass', activate_hour_glass)

            # sag_output  = { 'color': [gauge_color], 'mart': {'Phase 1':sag_count[0] , 'Phase 2':sag_count[1] ,  'Phase 3':sag_count[2] , 
            #                 'red_threshold': red_threshold_sag, 'yellow_threshold': yellow_threshold_sag, 'trendYMax': 500 , 'trendYMin': 0},# 'Message': -999},                                  
            #                 'success': 'true',  "reason": text_under_gauge, 'rank': 13, 'trendExistence': True, 
            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True, 
            #                 'mro': True, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow', 'red'],  
            #                 'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
            
        else:
            self.gauge_color = ["black"]
            # self.mart = {'Phase 1':-999 , 'Phase 2':-999 ,  'Phase 3':-999 , 
            #                 'red_threshold': red_threshold_sag, 'yellow_threshold': yellow_threshold_sag, 'trendYMax': 500 , 'trendYMin': 0}
            self.kpis = {'Phase 1':-999 , 'Phase 2':-999 ,  'Phase 3':-999}
            self.success = False

            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, gauge_name, self.Machine_status, [])
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]

            # sag_output  = {'color': ['black'],'mart': {'Phase 1':-999 , 'Phase 2':-999 ,  'Phase 3':-999 , 
            #                 'red_threshold': red_threshold_sag, 'yellow_threshold': yellow_threshold_sag, 'trendYMax': 500 , 'trendYMin': 0},# 'Message': "Machine Idle"},                                  
            #                 'success': 'false',  "reason": "Machine Idle", 'rank': 13, 'trendExistence': True, 
            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True, 
            #                 'mro': True, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow', 'red'],  
            #                 'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}

        # self.update_historical_ts()
        # self.add_gauge_alarm(sag_output,gauge_name,current_timestamp,self.machineid,self.subassembly_instance)
        # print('\n_____________ sag ________________\n')

        # return sag_output      
