
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *
from powercompute_utils import compute_voltage_imbalance, compute_current_imbalance

class GaugeStatorWindingFailure(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'stator_winding_failure'
    __pm_function_name__ = 'Stator Winding Failure'

    __rules__ = [
        {
           'name': 'red_threshold',
           'type': 'float'
        }

    ]

    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_stator_winding_failure'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Stator Winding Failure'
        self.gauge_name = 'Stator Winding Failure'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 11
        self.gaugeLife = 3600
        self.trendExistence = True
        self.mro = True
        self.feedback = True
        self.userInput = True
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdinput' else True
        self.reliability_quotient_output = 100
        self.feedback_colors = ["green","red"]

        self.hasHourglass = True
        self.hasAlarm = True
        self.trends = {"trendYMax" : 100, "trendYMin" : 0}
        self.units = "count"

    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        p["gaugePropertiesOutput"] = { "reliabilityQuotient" : self.reliability_quotient_output }
        return p


    def compute(self):
        current_timestamp = self.stored_object.current_ts()

        x = compute_current_imbalance(self.i_3phase)
        self.i_imbalance = x['i_imbalance']
        x = compute_voltage_imbalance(self.v_3phase)
        self.v_imbalance = x['imbalance']

        pf_range = max(self.pf_mean) - min(self.pf_mean)

        stator_red_threshold = self.get_gauge_rule_value("red_threshold",15) # This isn't actually used for a threshold just given in the gauge output.

        # stator_input_red_threshold = get_threshold(self.API_list,self.subassembly_instance, 'Stator Winding Failure', 'red_threshold' )
        # stator_red_threshold = 15 if stator_input_red_threshold ==None else stator_input_red_threshold

        # threshold_changed_indicator = 0 if stator_input_red_threshold== None else 1

        stator_winding_failure_color = 'red' if self.i_imbalance > 15 and self.v_imbalance <8 and pf_range > 0.5 else 'green'

        # trendYMax = 100  
        # trendYMin = 0

        # stator_winding_failure_indicator = [stator_winding_failure_color]
        # red_indicator = [1] if stator_winding_failure_color is 'red' else [0]
        # yellow_indicator = [0] # red_yellow_event_indicator('yellow', current_imbalance_indicator)
        # event_indicator = min(red_indicator[0] + yellow_indicator[0], 1)        

        # self.stored_object.current()[self.subassembly_instance].update({self.gauge_name:{'events':event_indicator, 'y_indicator':yellow_indicator, 'r_indicator':red_indicator }})
        self.mart_extras = {"red_threshold": stator_red_threshold}
        if self.Machine_status != 0:
            self.gauge_color        = stator_winding_failure_color
            # last_known_success = {'indicator': self.Machine_status,  'time': current_timestamp, 'color': self.gauge_color }
            # self.stored_object.current()[self.subassembly_instance].update({self.gauge_name:{'last_known_success': last_known_success}})

            # #print('\n before hourglass function')
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, self.gauge_name, self.Machine_status, [self.i_imbalance])
            # #print('\n after hourglass function: ', hourglass_color_value_reason_call)
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]
            # i_imbalance   		= hourglass_color_value_reason_call[2]  # value
            # reason              = hourglass_color_value_reason_call[3]

            self.success = True
            # self.mart = { "trend" : self.i_imbalance, "red_threshold": stator_red_threshold, "trendYMax" : trendYMax, "trendYMin" : trendYMin}
            self.kpis = { "trend" : self.i_imbalance }
            # stator_winding_failure_output=  { "color" : [ gauge_color ], "mart" :{ "trend" : i_imbalance[0], "red_threshold": stator_red_threshold, "trendYMax" : trendYMax, "trendYMin" : trendYMin} ,# "Message": -999 },
                                
            #                 "gaugePropertiesOutput" : { "reliabilityQuotient" : self.reliability_quotient_output }, "success" : "true",'reason':reason,
                        
            #         'rank': 11, 'trendExistence': True, 

            #         'hideFromGroup': False if self.subassembly.lower() != 'vfdinput' else True,

            #         'gaugeLife': 3600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'red'],  
            #                                         'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]} 

        else:  
            self.gauge_color = ['black']
            self.reason = "Machine Idle"
            self.reliabilityQuotent = -1
            # self.mart = { "trend" : -999, "red_threshold": stator_red_threshold, "trendYMax": trendYMax, "trendYMin": trendYMin}
            self.kpis = { "trend" : -999 }
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, self.gauge_name, self.Machine_status, [])
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]

            # stator_winding_failure_output = {'color': ['black'], "mart" : { "trend" : -999, "red_threshold": stator_red_threshold, "trendYMax": trendYMax, "trendYMin": trendYMin} ,# "Message": "Machine Idle" },  
            #                                 "gaugePropertiesOutput" : { "reliabilityQuotient" : -1 },
                                
            #                 "success" : "false", "reason" : "Machine Idle", 'rank': 11, 'trendExistence': True, 

            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdinput' else True, 

            #                 'gaugeLife': 3600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'red'],  
            #                                         'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}

        # self.update_historical_ts()
        # self.add_gauge_alarm(stator_winding_failure_output,self.gauge_name,current_timestamp,self.machineid,self.subassembly_instance)
        # return stator_winding_failure_output


