
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *

class GaugeSwell(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'swell'
    __pm_function_name__ = 'Swell'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]

    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_swell'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Swell'
        self.gauge_name = 'Swell'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 14
        self.gaugeLife = 600
        self.trendExistence = True
        self.mro = True
        self.feedback = True
        self.userInput = True
        self.hideFromGroup = False if self.subassembly.lower() != 'vfdoutput' else True
        self.hasHourglass = True
        self.hasAlarm = True
        self.trends = {"trendYMax" : 500, "trendYMin" : 0}
        self.units = "count"

    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']},"events": {"yellow":{},"red":{}},'batch_v_min':{}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p

    def get_worst_event_info(self):
        return self.get_worst_event_data_max()

    def get_worst_event_message(self):
        worst_event_info = self.get_worst_event_info()
        p_u_value = round(100*(1-worst_event_info['event'][0]),3)
        last_worst_event_time = worst_event_info['ts']
        last_event_time_format = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(int(last_worst_event_time)) )
        worst_event_message  = ';  Worst Event: Phases' + event_phases(worst_event_info['event'][-1]) + ', P.U.(%): ' + str(p_u_value) + ',  Time: ' + str(last_event_time_format) + ' UTC'
        return worst_event_message

    def compute(self):
        # current_timestamp = self.current_datetime_stamp
        #swell_count = [swell_count_ph1, swell_count_ph2, swell_count_ph3]
        #swell_count = [min_10_list[21], min_10_list[22], min_10_list[23]]
        max_swell_count = max(self.swell_count)
            
        #sag_swell_mag_max = min_10_list[32]
        
        yellow_threshold_swell = self.get_gauge_rule_value("yellow_threshold",1)
        red_threshold_swell = self.get_gauge_rule_value("red_threshold",3) 

        # input_yellow_threshold_swell   = get_threshold(self.API_list,self.subassembly_instance, 'swell', "yellow_threshold"  )
        # input_red_threshold_swell      = get_threshold(self.API_list,self.subassembly_instance, 'swell', "red_threshold"  )
        
        # yellow_threshold_swell         = 1 if input_yellow_threshold_swell == None else input_yellow_threshold_swell
        # red_threshold_swell            = 3 if input_red_threshold_swell == None else input_red_threshold_swell
        
        swell_color = 'red' if max_swell_count >= red_threshold_swell else 'yellow' if  max_swell_count >= yellow_threshold_swell else 'green'
            
        swell_color_indicator = list(map(lambda x: 'red' if x >= red_threshold_swell else 'yellow' if x >= yellow_threshold_swell else 'green', self.swell_count))
        # red_indicator    = red_yellow_event_indicator('red', swell_color_indicator)
        # yellow_indicator = red_yellow_event_indicator('yellow', swell_color_indicator)
        # event_indicator  = min(red_indicator[0] + yellow_indicator[0], 1)
        #print('\n sag color indicator: ', sag_color_indicator)
        #print('\n red indicator: ', red_indicator)
        #print('\n yellow indicator: ', yellow_indicator)
        #print('\n event indicator: ', event_indicator)
        
        
        batch_min = max(self.sag_swell_mag_max)
        swell_indicator  = calc_indicator(self.sag_swell_mag_max,batch_min)

        # self.stored_object.current()[self.subassembly_instance].update({self.gauge_name:{'events':event_indicator, 'y_indicator':yellow_indicator, 'r_indicator':red_indicator,'batch_v_min': swell_indicator }})
        self.update_indicators(swell_color_indicator,extras={'batch_v_min': swell_indicator})   
        #print('\n stored_obj: ', self.stored_object.current()[subassembly_instance] )
        
        self.mart_extras = {'red_threshold': red_threshold_swell, 'yellow_threshold': yellow_threshold_swell}
        if self.Machine_status != 0:
            self.gauge_color        = swell_color
            # last_known_success = {'indicator': self.Machine_status,  'time': current_timestamp, 'color': self.gauge_color }
            # self.stored_object.current()[self.subassembly_instance][self.gauge_name].update({'last_known_success': last_known_success})
            
            # self.reason = self.stored_object.worst_event_text(self.gauge_name, self.subassembly_instance)
            self.reason = self.get_worst_event_text()
            #print('\n text under gauge: ', text_under_gauge)

            # #print('\n before hourglass function')
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, self.gauge_name, self.Machine_status, self.swell_count)
            # #print('\n after hourglass function: ', hourglass_color_value_reason_call)
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]
            # swell_count   = hourglass_color_value_reason_call[2]  # value
            # reason              = hourglass_color_value_reason_call[3]
                
            # reason              =reason if reason != '' else text_under_gauge
            # #print('\n activate hour glass', activate_hour_glass)

            self.success = True
            # self.mart = {'Phase 1':self.swell_count[0] , 'Phase 2':self.swell_count[1] ,  'Phase 3':self.swell_count[2] , 
            #                 'red_threshold': red_threshold_swell, 'yellow_threshold': yellow_threshold_swell, 'trendYMax': 500 , 'trendYMin': 0}
            self.kpis = {'Phase 1':self.swell_count[0] , 'Phase 2':self.swell_count[1] ,  'Phase 3':self.swell_count[2]}
            # swell_output  = { 'color': [gauge_color], 'mart': {'Phase 1':swell_count[0] , 'Phase 2':swell_count[1] ,  'Phase 3':swell_count[2] , 
            #                 'red_threshold': red_threshold_swell, 'yellow_threshold': yellow_threshold_swell, 'trendYMax': 500 , 'trendYMin': 0},# 'Message': -999},                                  
            #                 'success': 'true',  "reason": text_under_gauge, 'rank': 14, 'trendExistence': True, 
            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True,  
            #                 'mro': True, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow', 'red'],  
            #                 'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
            
        else:
            self.reason = "Machine Idle"
            self.gauge_color = ["black"]
            # self.mart = {'Phase 1':-999, 'Phase 2':-999 ,  'Phase 3':-999 , 
            #                 'red_threshold': red_threshold_swell, 'yellow_threshold': yellow_threshold_swell, 'trendYMax': 500 , 'trendYMin': 0}
            self.kpis = {'Phase 1':-999, 'Phase 2':-999 ,  'Phase 3':-999}
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, self.gauge_name, self.Machine_status, [])
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # gauge_color         = hourglass_color_value_reason_call[1]

            # swell_output  = { 'color': ['black'],'mart': {'Phase 1':-999, 'Phase 2':-999 ,  'Phase 3':-999 , 
            #                 'red_threshold': red_threshold_swell, 'yellow_threshold': yellow_threshold_swell, 'trendYMax': 500 , 'trendYMin': 0},# 'Message': "Machine Idle"},                                  
            #                 'success': 'false',  "reason": "Machine Idle", 'rank': 14, 'trendExistence': True, 
            #                 'hideFromGroup': False if self.subassembly.lower() != 'vfdoutput' else True,  
            #                 'mro': True, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow', 'red'],  
            #                 'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}

        # self.update_historical_ts()

        # self.add_gauge_alarm(swell_output,self.gauge_name,current_timestamp,self.machineid,self.subassembly_instance)
            
        # print('\n_____________after swell  ________________\n')
        # return swell_output

