
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeColor
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from powercompute_utils import compute_voltage_imbalance
class GaugeVoltageImbalance(GaugePowerAnalyzerMixin,GaugeTypeColor):
    __internal_gauge_name__= 'voltage_imbalance'
    __pm_function_name__ = 'Voltage Imbalance'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]


    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_voltage_imbalance'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Voltage Imbalance'
        self.gauge_name = 'Voltage Imbalance'
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = "PF"
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 12
        self.gaugeLife = 600
        self.trendExistence = True
        self.feedback = True
        self.mro = True
        self.hasAlarm = True
        self.hasHourglass = True
        self.userInput = True
        self.trends = {"trendYMax" : 100, "trendYMin" : 0}
        self.units = "%"


    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']},"events": {"yellow":{},"red":{}}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p

    def compute(self):
        '''
        VOLTAGE IMBALANCE___________
        '''

        # current_timestamp = self.current_datetime_stamp
        # gauge_name = 'Voltage Imbalance'
        
        # try:
        #     print("\n 3phase voltage {}  and  average voltage {}\n".format(self.v_3phase,self.v_avg))
        #     if self.v_avg !=0 : #and v_3phase != np.array([0, 0, 0]):
                    
        #         #v_deviations  = [100* (abs(float(f)-float(v_avg))/float(v_avg) for f in v_3phase)]#[100/v_avg * (abs(f-v_avg) for f in v_3phase)] # 100/v_avg * abs(v_3phase- v_avg)
                    
        #         v_deviations=[]
        #         for v_phase_t in self.v_3phase:
        #             v_deviations.append( abs ( (v_phase_t-self.v_avg) * 100/self.v_avg  ) )
                    
        #         #print(v_deviations)
        #         self.v_imbalance   = max(v_deviations)
        #         #print("\n v_avg !=0  and  3phase values !=0 \n")
        #     else:
        #         v_deviations= np.array([0, 0, 0])
        #         self.v_imbalance=0  
        #         #print("\n v_avg =0  OR  3phase values =0 \n")              
        # except:
        #     v_deviations= np.array([0, 0, 0])
        #     self.v_imbalance=0
        #     #print("\n v_avg =0  OR  unable to compute 3phase \n")

        x = compute_voltage_imbalance(self.v_3phase)
        v_deviations = x['deviations']
        self.v_imbalance = x['imbalance']

        yellow_threshold = self.get_gauge_rule_value("yellow_threshold",2)
        red_threshold = self.get_gauge_rule_value("red_threshold",4) 

        # input_yellow_threshold       = get_threshold(self.API_list,self.subassembly_instance, 'voltage imbalance', "yellow_threshold"  ) 
        # yellow_threshold             = 2 if input_yellow_threshold == None else input_yellow_threshold
        
        # input_red_threshold          = get_threshold(self.API_list,self.subassembly_instance, 'voltage imbalance', "red_threshold"  )
        # red_threshold                =  4 if input_red_threshold == None else input_red_threshold
        
        self.gauge_color      = 'red' if self.v_imbalance >= red_threshold else 'yellow' if  self.v_imbalance >= yellow_threshold else 'green'
            
        v_imbalance_indicator = list(map(lambda x: 'red' if x >= red_threshold else 'yellow' if x >= yellow_threshold else 'green', v_deviations))
        print("----------------------- yellow/red thresholds: ", yellow_threshold,red_threshold)

        # red_indicator    = red_yellow_event_indicator('red', v_imbalance_indicator)
        # yellow_indicator = red_yellow_event_indicator('yellow', v_imbalance_indicator)
        # event_indicator  = min(red_indicator[0] + yellow_indicator[0], 1)

        #print('\n after event_indicator: ', event_indicator)
        #print('\n current timestamp on V imb: ', current_timestamp)
        #print('\n type of current time: ', type(current_timestamp))
            
        # for a, b in self.stored_object.items():
        #     if a== current_timestamp:
        #         #print('\n printing stored obj of this timestamp: ',stored_object_within_24hrs.current())
        #         pass
                    
        # self.stored_object.current()[self.subassembly_instance].update({gauge_name:{'events':event_indicator, 'y_indicator':yellow_indicator, 'r_indicator':red_indicator }})
        #print('\n printing stored obj in v_imb after stored obj processing', stored_object_within_24hrs.current()[subassembly_instance])
        self.update_indicators(v_imbalance_indicator)    

        self.mart_extras = {'red_threshold': red_threshold, 'yellow_threshold': yellow_threshold}

        if self.Machine_status != 0:
            #print('\n if machine status is not none: ')
            #print(Machine_status)

            # gauge_color        = self.gauge_color
            # last_known_success = {'indicator': self.Machine_status,  'time': current_timestamp, 'color': gauge_color }
            # self.stored_object.current()[self.subassembly_instance][gauge_name].update({'last_known_success': last_known_success})
            #print('\n stored_obj for last known success: ', stored_object_within_24hrs.current()[subassembly_instance][gauge_name])

            # self.reason = self.stored_object.get_text_under_gauge('Voltage Imbalance', self.subassembly_instance)
            self.reason = self.get_text_under_gauge()
    
            #print('\n printing text_under_gauge: ', text_under_gauge)

            #print('\n before hourglass function')
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, gauge_name, self.Machine_status, v_deviations)
            #print('\n after hourglass function: ', hourglass_color_value_reason_call)
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # self.gauge_color         = hourglass_color_value_reason_call[1]
            # v_deviations   = hourglass_color_value_reason_call[2]  # value
            # reason              = hourglass_color_value_reason_call[3]
                
            # self.reason              =reason if reason != '' else text_under_gauge
            #print('\n activate hour glass', activate_hour_glass)

            self.success = True
            # self.mart = {
            #     'Phase 1':v_deviations[0] , 
            #     'Phase 2':v_deviations[1] ,  
            #     'Phase 3':v_deviations[2] , 
            #     'red_threshold': red_threshold, 
            #     'yellow_threshold': yellow_threshold, 
            #     'trendYMax': 100 , 
            #     'trendYMin': 0
            # }
            self.kpis={'Phase 1':v_deviations[0] , 'Phase 2':v_deviations[1] ,  'Phase 3':v_deviations[2]}
            # gauge_output = { 'color': [gauge_color], 'mart': {'Phase 1':v_deviations[0] , 'Phase 2':v_deviations[1] ,  'Phase 3':v_deviations[2] , 
            #                         'red_threshold': red_threshold, 'yellow_threshold': yellow_threshold, 'trendYMax': 100 , 'trendYMin': 0},# 'Message': -999},                                  
            #                         'success': 'true',  "reason": text_under_gauge, 'rank': 12, 'trendExistence': True, 'hideFromGroup': False, 'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow', 'red'],  
            #                         'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
                                    
            #print('\n printing v_imba_output: ', gauge_output)
        else:
            # hourglass_color_value_reason_call = self.stored_object.last_known_success_process(self.subassembly_instance, gauge_name, self.Machine_status, [])
            # activate_hour_glass = hourglass_color_value_reason_call[0]
            # self.gauge_color         = hourglass_color_value_reason_call[1]
            # self.mart = {
            #     'Phase 1':-999 , 
            #     'Phase 2':-999 ,  
            #     'Phase 3':-999 , 
            #     'red_threshold': red_threshold, 
            #     'yellow_threshold': yellow_threshold, 
            #     'trendYMax': 100 , 
            #     'trendYMin': 0
            # }
            self.kpis = {'Phase 1':-999 , 'Phase 2':-999 ,  'Phase 3':-999 }
            self.reason = "Machine Idle"
            self.gauge_color = ["black"]
            # gauge_output = { 'color': ['black'],'mart': {'Phase 1':-999 , 'Phase 2':-999 ,  'Phase 3':-999 , 
            #                         'red_threshold': red_threshold, 'yellow_threshold': yellow_threshold, 'trendYMax': 100 , 'trendYMin': 0},# 'Message': 'Machine Idle'},                                  
            #                         'success': 'false',  "reason": "Machine Idle", 'rank': 12, 'trendExistence': True, 'hideFromGroup': False,  'gaugeLife': 600, 'alarmYellowIssueOpen': [False], 'sendYellowAlertHistory': [False], 'sendYellowAlert': [False], 'sendRedAlert': [False], 'alarmRedIssueOpen': [False], 'sendRedAlertHistory': [False], 'feedbackColors': ['green', 'yellow', 'red'],  
            #                         'mro': True, 'feedback': True, 'activateHourGlass': activate_hour_glass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': self.subassembly_instance, 'subassembly': self.subassembly, 'userInput': True, "publishingSensorType": "PF", 'alarmRedIssueOpenHistory': [False], 'alarmYellowIssueOpenHistory': [False], 'value': [-1]}
        
        # self.update_historical_ts()
        #self.add_gauge_alarm(gauge_output,gauge_name,current_timestamp,self.machineid,self.subassembly_instance)
        print('\n_____________after v imb________________\n')
        #return gauge_output 


