
from pynovatec_compute.gauges import GaugeMissingData, GaugeTypeBar
from GaugePowerAnalyzerMixin import GaugePowerAnalyzerMixin
from Additional_functions import *

class GaugeVoltageThd(GaugePowerAnalyzerMixin,GaugeTypeBar):
    __internal_gauge_name__= 'voltage_thd'
    __pm_function_name__ = 'Voltage Harmonic Distortion'

    __rules__ = [
        {
           'name': 'yellow_threshold',
           'type': 'float'
        },
        {
           'name': 'red_threshold',
           'type': 'float'
        }
    ]

    def __init__(self,*args,**kwargs):
        kwargs['internal_gauge_name'] = '__test_gauge_voltage_thd'
        super().__init__(*args,**kwargs)
        self.gauge_output_name = 'Voltage Harmonic Distortion'
        self.gauge_name = 'Voltage Harmonic Distortion'
        self.activateHourGlass = True if self.Machine_status== 0 else False
        self.pre_yellow_threshold = 80;
        self.publishing_sensors = 'PF'
        self.pre_red_threshold = 95
        self.default_thresholds = {'yellow': 80, 'red': 95}
        self.rank = 41
        self.trendExistence = True
        self.gaugeLife = 600
        self.userInput = True
        self.hasAlarm = False   # Looks like it doesn't use alarm, so this isn't needed.
        self.trends = {"trendYMax" : 100, "trendYMin" : 0}
        self.units = "%"
        self.gauge_color = ['white'] * 3

    def get_default_stored_object(self):
            return {'last_known_success':{'true':0,'false':0, 'color':['white']}}

    def get_properties(self):
        p = super().get_properties()
        p['type'] = self.gauge_type
        return p


    def compute(self):
        '''
        VOLTAGE THD________________
        '''
        
        gauge_name   =  ['Phase-1', 'Phase-2', 'Phase-3']  
        
        yellow_threshold = self.get_gauge_rule_value("yellow_threshold",5)
        red_threshold = self.get_gauge_rule_value("red_threshold",7) 

        # input_yellow_threshold       = get_threshold(self.API_list,self.subassembly_instance, 'voltage harmonic distortion', "yellow_threshold"  ) 
        # yellow_threshold             = 5 if input_yellow_threshold == None else input_yellow_threshold
        
        # input_red_threshold          = get_threshold(self.API_list,self.subassembly_instance, 'voltage harmonic distortion', "red_threshold"  )
        # red_threshold                = 7 if input_red_threshold == None else input_red_threshold
        
        self.gauge_color           = list(map(lambda x: 'red' if x >= red_threshold else 'yellow' if  x >= yellow_threshold else 'green', self.vthd))
        # self.mart = {'Phase 1': self.vthd[0], 'Phase 2': self.vthd[1], 'Phase 3': self.vthd[2], 'trendYMax': 100, 'trendYMin': 0}
        self.kpis = {'Phase 1': self.vthd[0], 'Phase 2': self.vthd[1], 'Phase 3': self.vthd[2]}
        self.success = True

        # vthd_output          = { 'name': gauge_name, 'value': vthd, 'color': vthd_color, 'type': 'bar',                
        #                         'mart': {'Phase 1': vthd[0], 'Phase 2': vthd[1], 'Phase 3': vthd[2], 'trendYMax': 100, 'trendYMin': 0},# 'Message': -999}, 
        #                         'success': 'true', 'rank': 41, 'trendExistence': True, 
        #                         'hideFromGroup': False if subassembly.lower() != 'vfdoutput' else True,  
        #                         'gaugeLife': 600, 
        #                         'alarmYellowIssueOpen': [False, False, False], 
        #                         'sendYellowAlertHistory': [False, False, False], 
        #                         'sendYellowAlert': [False, False, False], 
        #                         'sendRedAlert': [False, False, False], 
        #                         'alarmRedIssueOpen': [False, False, False], 
        #                         'sendRedAlertHistory': [False, False, False],  
        #                         'mro': False, 'feedback': False, 'activateHourGlass': activateHourGlass, 'publishingSensorType': 'PF', 'dummyGauge': False, 'subassemblyInstance': subassembly_instance, 'subassembly': subassembly, 'userInput': True, "publishingSensorType": "PF", 
        #                         'alarmRedIssueOpenHistory': [False,False, False], 'alarmNames': ["Phase 1", "Phase 2","Phase 3"], 
        #                         'alarmYellowIssueOpenHistory': [False,False, False]}
                
        # print('\n_____________after v thd ________________\n')
        # return vthd_output


