 # -*- coding: utf-8 -*-
"""
Created on Sun Nov 10 18:35:24 2019

@author: ANIK SARDAR
"""


import os, time
import simplejson as json
from os import path
from simplenumpyencoder import NumpyEncoder
from ZmqReceiver import ZmqReceiver
import appconfig
from Additional_functions import *
import getopt
import sys
from pynovatec_compute.objectstore import ObjectStore
from pynovatec_compute.loggers import JsonLogger, LoggerInit,log_save,log_purge
from pynovatec_compute.gauges import loadGaugeDetails
from poweranalyzer_compute import PowerAnalyzerOneMinBatchProcessor, PowerAnalyzerTenMinBatchProcessor
from pynovatec_compute.streamers import ZmqStreamer, JsonFileStreamer
import traceback

default_config = {
    'data_directory':'/opt/novatec/data/pumpsense/py_process', 
    #'macid':system( "sudo ifconfig eth0 | awk '/HWaddr/ {print $NF}' ", intern = T ),   #... to get mac id from rpi                
    'mainDir':'/opt/novatec/data/pumpsene',
    'json_details_path':'/opt/novatec/data/pumpsense/json_data/',
    'stored_objects_directory':'/opt/novatec/data/pumpsense/py_process/stored_object',
    'product':'es',
    'build':'production',
    'json_receiver_connect_string':"tcp://*:5556",
    'json_sender_connect_string':'tcp://localhost:4445',
    'input_json_dir': '/opt/novatec/data/pumpsense/py_process/input_json/',
    'output_json_dir': '/opt/novatec/data/pumpsense/py_process/output_json/',
    'purge_frequency': 60,
    'log_file': "/tmp/reason.log",
    'output_format': 1,
    'include_vmart': True
}

def make_config_directory(dirname):
    if not path.exists(dirname):
        os.makedirs(dirname) 

def setup_directories():
    make_config_directory(appconfig.data_directory)
    make_config_directory(appconfig.input_json_dir)
    make_config_directory(appconfig.output_json_dir)
    make_config_directory(appconfig.stored_objects_directory)

    input_1_dir = os.path.join(appconfig.input_json_dir,"INPUT_1")
    input_10_dir = os.path.join(appconfig.input_json_dir,"INPUT_10")
    output_1_dir = os.path.join(appconfig.output_json_dir,"OUTPUT_1")
    output_10_dir = os.path.join(appconfig.output_json_dir,"OUTPUT_10")
    # make_config_directory(input_1_dir)
    # make_config_directory(input_10_dir)
    # make_config_directory(output_1_dir)
    # make_config_directory(output_10_dir)
    appconfig.set('input_1_dir',input_1_dir)
    appconfig.set('input_10_dir',input_10_dir)
    appconfig.set('output_1_dir',output_1_dir)
    appconfig.set('output_10_dir',output_10_dir)

appconfig.add(**default_config)

def initProcessing():
    global appconfig
    global input_json_receiver

    if hasattr(appconfig,"config_file") and path.exists(appconfig.config_file):
        print('\n if config file exists \n')
        with open (appconfig.config_file) as f:
            config_info= json.load(f)
        appconfig.add(**config_info)
    
    product              = appconfig.product.lower()
    effective_build_name = appconfig.build.lower()
    ObjectStore(appconfig.stored_objects_directory)
    LoggerInit(appconfig.log_file)

    print('\n after reading the config_file_path and, get product and build name \n')
    #    jena = 0
    setup_directories()        
    try:
        os.chdir(appconfig.data_directory)  # local
    except Exception as e:
        print(f"Couldn't change directories to: {appconfig.data_directory}")
        print(str(e))
    gauge_details_json_path = os.path.join(appconfig.json_details_path,'gauge_details_0.json')
    loadGaugeDetails(gauge_details_json_path,metas={'kFactor': 1, 'cttype':100})


def initZmq():
    global input_json_receiver

    input_json_receiver = ZmqReceiver(appconfig.json_receiver_connect_string)


def StoreInput(mins_info,input_json,csv_file_name,machineid):
    pass


last_purge_timestamp=0

def PurgeFiles(timestamp):
    global last_purge_timestamp

    now = int(time.time())
    if now - last_purge_timestamp > appconfig.purge_frequency:
        print("************ PURGING ******************")
        AutoDeletion(appconfig.output_1_dir, 3600, timestamp)
        AutoDeletion( appconfig.input_1_dir, 3600, timestamp)
        AutoDeletion_10(appconfig.output_10_dir, 14400, timestamp)
        AutoDeletion_10( appconfig.input_10_dir, 14400, timestamp)
        last_purge_timestamp = timestamp


def main_loop():
    global input_json_receiver
    global OneMinBatchProcess
    global TenMinBatchProcess
    while True:                     # replacement for repeat loop in R

        try:
            print(' \n under the try block while == True, before selecting the systemct_os \n')                                             

            ############## following part is reading from socket done on rpi ##################
            #---------------------------------------------------------------------------------
            result = input_json_receiver.recv_batch_input_json()
            mins_info = result['mins_info']
            json0 = result['json']
            print('\n mins_info: ',mins_info)
#            print('\n printing json0: \n',json0)
            #--------------------------------------------------------------------------------
            ##################### till here input json reading from socket ###################

            print('\n after json0 parsing is done \n')
            if len(json0)!=0:
                input_json    = json.loads(json0) # executed when json read from socket
                if mins_info=='10':
                    TenMinBatchProcess.processBatch(input_json, output_format=appconfig.output_format, include_vmart=appconfig.include_vmart)
                else:
                    OneMinBatchProcess.processBatch(input_json, output_format=appconfig.output_format, include_vmart=appconfig.include_vmart)
                log_purge()
                log_save()
        except Exception as e:
            print(e)
            print(traceback.format_exc())

create_gauge_rules=None

def setup():
    global create_gauge_rules

    try:
        opts, args = getopt.getopt(sys.argv[1:],"c:R:")
    except getopt.GetoptError:
        print("Error: Error Loading Configuration Details: ")
    for opt,arg in opts:
        #print("opt is: ",opt)
        if opt == "-c":
            appconfig.config_file = arg
        if opt == "-R":
            create_gauge_rules = arg

from GenGaugeRules import gen_all_gauge_rules

def doCreateGaugeRules(doExit=False):
    global create_gauge_rules

    if create_gauge_rules:
        gen_all_gauge_rules(appconfig.json_details_path,dest_file=create_gauge_rules)
    else:
        gen_all_gauge_rules(appconfig.json_details_path)
    if doExit:
        sys.exit()

setup()
initProcessing()

if create_gauge_rules:
    doCreateGaugeRules(doExit=True)
initZmq()
#appconfig.dump()

StreamingZMQ = ZmqStreamer(appconfig.json_sender_connect_string,options={'cls': NumpyEncoder})

OneMinInputStreamers = [JsonFileStreamer(appconfig.input_1_dir)]
OneMinOutputStreamers = [JsonFileStreamer(appconfig.output_1_dir),StreamingZMQ]
OneMinBatchProcess = PowerAnalyzerOneMinBatchProcessor(OneMinInputStreamers, OneMinOutputStreamers)

TenMinInputStreamers = [JsonFileStreamer(appconfig.input_10_dir)]
TenMinOutputStreamers = [JsonFileStreamer(appconfig.output_10_dir),StreamingZMQ]
TenMinBatchProcess = PowerAnalyzerTenMinBatchProcessor(TenMinInputStreamers, TenMinOutputStreamers)


main_loop()           
