# -*- coding: utf-8 -*-
"""
Created on Sun Nov 10 18:35:24 2019

@author: ANIK SARDAR
"""

'''
                                                 ________________                                                 
                                                 INPUT PROCESSING
                                                 ________________
'''
#________________________________________________________________________________________________________________________________
import numpy as np
from min_1_gauges import *
from min_10_gauges import *
from Additional_functions import *
from pynovatec_compute.gauges import get_machine_rules
import traceback

def get_sensor_data_values(json,source):
    if source not in json["collectors"]["pf1"]:
        return np.array([])
    u = json["collectors"]["pf1"][source][0]
    value_list    = np.array([[float(x) for x in f[1:]] for f in u])
    return value_list

def get_np_sensor_data_values(json,source):
    vl = get_sensor_data_values(json,source)
    return np.array(vl)

def get_sensor_time_values(json,source):
    if source not in json["collectors"]["pf1"]:
        return []
    u = json["collectors"]["pf1"][source][0]
    time_list = [[x for x in f[0]] for f in u]
    return time_list

def get_harmonic_norm(value_list_10,PF_3Phase_medians):
    phase_i_csv10 = phase_wise(value_list_10)

    print('\n @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n')
    df10_ph1 = phase_i_csv10[0]
    df10_ph2 = phase_i_csv10[1]
    df10_ph3 = phase_i_csv10[2]
    print('\n !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n')
    # EXTRACTING HARMONICS DATA FOR PHASE 1
    h1_ph1  = np.mean(df10_ph1[:,1])
    h2_ph1  = np.mean(df10_ph1[:,2])
    h3_ph1  = np.mean(df10_ph1[:,3])
    h4_ph1  = np.mean(df10_ph1[:,4])
    h5_ph1  = np.mean(df10_ph1[:,5])
    h6_ph1  = np.mean(df10_ph1[:,6])
    h7_ph1  = np.mean(df10_ph1[:,7])
    h8_ph1  = np.mean(df10_ph1[:,8])
    h9_ph1  = np.mean(df10_ph1[:,9])
        
    #crest_factor_ph =  np.mean(df10_ph1[:,10])
    #fund_freq_ph    =  np.mean(df10_ph1[:,11])

    # EXTRACTING HARMONICS DATA FOR PHASE 2
    h1_ph2  = np.mean(df10_ph2[:,1])
    h2_ph2  = np.mean(df10_ph2[:,2])
    h3_ph2  = np.mean(df10_ph2[:,3])
    h4_ph2  = np.mean(df10_ph2[:,4])
    h5_ph2  = np.mean(df10_ph2[:,5])
    h6_ph2  = np.mean(df10_ph2[:,6])
    h7_ph2  = np.mean(df10_ph2[:,7])
    h8_ph2  = np.mean(df10_ph2[:,8])
    h9_ph2  = np.mean(df10_ph2[:,9])
    
    # EXTRACTING HARMONICS DATA FOR PHASE 3
    h1_ph3  = np.mean(df10_ph3[:,1])
    h2_ph3  = np.mean(df10_ph3[:,2])
    h3_ph3  = np.mean(df10_ph3[:,3])
    h4_ph3  = np.mean(df10_ph3[:,4])
    h5_ph3  = np.mean(df10_ph3[:,5])
    h6_ph3  = np.mean(df10_ph3[:,6])
    h7_ph3  = np.mean(df10_ph3[:,7])
    h8_ph3  = np.mean(df10_ph3[:,8])
    h9_ph3  = np.mean(df10_ph3[:,9])
    
    ph1_norm = (h3_ph1**2 + h5_ph1**2 + h7_ph1**2)**.5
    ph2_norm = (h3_ph2**2 + h5_ph2**2 + h7_ph2**2)**.5
    ph3_norm = (h3_ph3**2 + h5_ph3**2 + h7_ph3**2)**.5
    harmonic_data       = [ph1_norm, ph2_norm, ph3_norm]
    power_factor_median = max(PF_3Phase_medians)
    harmonic_median     = np.median(harmonic_data)
    harmonic_norm       = [harmonic_median, power_factor_median]
    return harmonic_norm


def extract_means(value_list, index_list):
    l=[]
    for i in index_list:
        r = np.mean(value_list[:,i])
        l.append(r)
    return l

def extract_maxes(value_list, index_list):
    l=[]
    for i in index_list:
        r = np.max(value_list[:,i])
        l.append(r)
    return l

def extract_mins(value_list, index_list):
    l=[]
    for i in index_list:
        r = np.min(value_list[:,i])
        l.append(r)
    return l

def extract_medians(value_list, index_list):
    l=[]
    for i in index_list:
        r = np.median(value_list[:,i])
        l.append(r)
    return l


def get_voltage_means(value_list):
    v1  = np.mean(value_list[:,0])
    v2  = np.mean(value_list[:,2])
    v3  = np.mean(value_list[:,4])

    return [v1,v2,v3]

def get_current_means(value_list, k_factor):
    i1  = np.mean(value_list[:,1])*k_factor        
    i2  = np.mean(value_list[:,3])*k_factor
    i3  = np.mean(value_list[:,5])*k_factor

    return [i1,i2,i3]

def get_av(current_means):
    i1 = current_means[0]
    i2 = current_means[1]
    i3 = current_means[2]

    av_1  = abs(1 - i1/i2) if i2!=0 else abs(1-i1)
    av_2  = abs(1 - i2/i3) if i3!=0 else abs(1-i2)
    av_3  = abs(1 - i3/i1) if i1!=0 else abs(1-i3)
    return [av_1,av_2,av_3]

def get_batch_voltage_max(value_list):
    v1_max = np.max(value_list[:,0])
    v2_max = np.max(value_list[:,2])
    v3_max = np.max(value_list[:,4])
    return [v1_max,v2_max,v3_max]

def get_batch_voltage_min(value_list):
    v1_min = np.min(value_list[:,0])
    v2_min = np.min(value_list[:,2])
    v3_min = np.min(value_list[:,4])
    return [v1_min,v2_min,v3_min]

def get_pf1_means(value_list):
    pf1_mean = np.mean(value_list[:,6])
    pf2_mean = np.mean(value_list[:,7])
    pf3_mean = np.mean(value_list[:,8])

    return [pf1_mean, pf2_mean, pf3_mean]


def get_pf1_max(value_list):
    pf1_max = np.max(value_list[:,6])
    pf2_max = np.max(value_list[:,7])
    pf3_max = np.max(value_list[:,8])

    return [pf1_max, pf2_max, pf3_max]

def get_pf1_min(value_list):
    pf1_min = np.min(value_list[:,6])
    pf2_min = np.min(value_list[:,7])
    pf3_min = np.min(value_list[:,8])

    return [pf1_min, pf2_min, pf3_min]

def get_pf1_median(value_list):
    pf1_median = np.median(value_list[:,6])
    pf2_median = np.median(value_list[:,7])
    pf3_median = np.median(value_list[:,8])

    return [pf1_median, pf2_median, pf3_median]

def calc_active_power(value_list,k_factor):
    active_power_1 = np.mean(value_list[:,0] * value_list[:,1] * value_list[:,6]) * k_factor / 1000
    active_power_2 = np.mean(value_list[:,2] * value_list[:,3] * value_list[:,7]) * k_factor / 1000
    active_power_3 = np.mean(value_list[:,4] * value_list[:,5] * value_list[:,8]) * k_factor / 1000
    return [active_power_1, active_power_2,active_power_3]

def get_voltage_thd_means(value_list_6):
    v1 = np.mean(value_list_6[:,0])
    v2 = np.mean(value_list_6[:,2])
    v3 = np.mean(value_list_6[:,4])
    return [v1,v2,v3]

def get_voltage_thd_medians(value_list_6):
    v1 = np.median(value_list_6[:,0])
    v2 = np.median(value_list_6[:,2])
    v3 = np.median(value_list_6[:,4])
    return [v1,v2,v3]


def get_current_thd_means(value_list_6):
    i1 = np.mean(value_list_6[:,1])
    i2 = np.mean(value_list_6[:,3])
    i3 = np.mean(value_list_6[:,5])
    return [i1,i2,i3]

def get_current_thd_medians(value_list_6):
    i1 = np.median(value_list_6[:,1])
    i2 = np.median(value_list_6[:,3])
    i3 = np.median(value_list_6[:,5])
    return [i1,i2,i3]

def get_sag_swell_means(value_list_6):
    sag_swell_ph1 = np.mean(value_list_6[:,6])
    sag_swell_ph2 = np.mean(value_list_6[:,7])
    sag_swell_ph3 = np.mean(value_list_6[:,8])
    return [sag_swell_ph1,sag_swell_ph2,sag_swell_ph3]

def get_sag_swell_maxes(value_list_6):
    sag_swell_ph1 = np.max(value_list_6[:,6])
    sag_swell_ph2 = np.max(value_list_6[:,7])
    sag_swell_ph3 = np.max(value_list_6[:,8])
    return [sag_swell_ph1,sag_swell_ph2,sag_swell_ph3]

def get_sag_swell_medians(value_list_6):
    sag_swell_ph1 = np.median(value_list_6[:,6])
    sag_swell_ph2 = np.median(value_list_6[:,7])
    sag_swell_ph3 = np.median(value_list_6[:,8])
    return [sag_swell_ph1,sag_swell_ph2,sag_swell_ph3]

def get_sag_swell_mins(value_list_6):
    sag_swell_ph1 = np.min(value_list_6[:,6])
    sag_swell_ph2 = np.min(value_list_6[:,7])
    sag_swell_ph3 = np.min(value_list_6[:,8])
    return [sag_swell_ph1,sag_swell_ph2,sag_swell_ph3]

def get_sag_counts(value_list_6):
    sag_count_ph1     = len(list(filter((lambda x: x<0.8), value_list_6[:,6])))
    sag_count_ph2     = len(list(filter((lambda x: x<0.8), value_list_6[:,7])))
    sag_count_ph3     = len(list(filter((lambda x: x<0.8), value_list_6[:,8])))
    return [sag_count_ph1,sag_count_ph2,sag_count_ph3]

def get_swell_counts(value_list_6):
    swell_count_ph1   = len(list(filter((lambda x: x>1.2), value_list_6[:,6])))
    swell_count_ph2   = len(list(filter((lambda x: x>1.2), value_list_6[:,7])))
    swell_count_ph3   = len(list(filter((lambda x: x>1.2), value_list_6[:,8])))
    return [swell_count_ph1,swell_count_ph2,swell_count_ph3]

def get_voltage_overruns(value_list,threshold):
    v1_overrun  = list(map(lambda x: 1 if x>threshold else 0, value_list[:,0]))
    v2_overrun  = list(map(lambda x: 1 if x>threshold else 0, value_list[:,2]))
    v3_overrun  = list(map(lambda x: 1 if x>threshold else 0, value_list[:,4]))
    return [v1_overrun,v2_overrun,v3_overrun]

def get_voltage_underruns(value_list,threshold):
    v1_underrun = list(map(lambda x: 1 if x<threshold else 0, value_list[:,0]))
    v2_underrun = list(map(lambda x: 1 if x<threshold else 0, value_list[:,2]))
    v3_underrun = list(map(lambda x: 1 if x<threshold else 0, value_list[:,4]))
    return [v1_underrun,v2_underrun,v3_underrun]


def make_min_1_dict(json, subassembly_instance,k_factor):
    value_list = get_sensor_data_values(json["subassemblies"][subassembly_instance],"CSV_4")
    if value_list.size == 0:
        return {
            'voltages': [],
            'currents': [],
            'pfs': [],
            'powers': []
        }
    voltage_means = extract_means(value_list,[0,2,4])
    current_means = get_current_means(value_list,k_factor)
    PF_3Phase_means = get_pf1_means(value_list)
    active_powers = calc_active_power(value_list,k_factor)
    active_energy_total = None
    apparent_energy_total = None
    if "CSV_20" in json["subassemblies"][subassembly_instance]["collectors"]["pf1"]:
        u_20  =  json["subassemblies"][subassembly_instance]["collectors"]["pf1"]["CSV_20"][0]
        time_list_20     = [[x for x in f[0]] for f in u_20]
        value_list_20    = np.array([[k_factor*float(x) for x in f[1:]] for f in u_20])

        active_energy_total    = value_list_20[:,0][len(value_list_20)-1] - value_list_20[:,0][0]
        apparent_energy_total  = value_list_20[:,1][len(value_list_20)-1] - value_list_20[:,1][0]

        #print(apparent_energy_total)
    else:
        active_energy_total=None
        apparent_energy_total=None

    min_1_dict = {
        'voltages': voltage_means,
        'currents': current_means,
        'pfs': PF_3Phase_means,
        'powers': active_powers,
        'active_energy_total': active_energy_total,
        'apparent_energy_total': apparent_energy_total
    }
    return min_1_dict


def preprocess_10min_data(json, subassembly_instance,k_factor):
    value_list = get_sensor_data_values(json["subassemblies"][subassembly_instance],"CSV_4")
    d={}
    if value_list.size > 0:
        print("Computing for value_list 4")
        d["Machine_running_percent_pf"] = utilization(value_list, 'Load Utilization')     # these two functions are called in 10 min gauges for on time and load utilization calculation
        d["Machine_running_percent_i"]  = utilization(value_list, 'Machine Utilization')

        # k_factor = machine_parameters[0]
        # cttype   = machine_parameters[1]

        d["voltage_means"] = extract_means(value_list,[0,2,4])
        d["current_means"] = get_current_means(value_list,k_factor)
        d["av"] = get_av(d["current_means"])
        d["batch_voltage_max"] = get_batch_voltage_max(value_list)   # included for overvoltage
        d["batch_voltage_min"] = get_batch_voltage_min(value_list)   # included for undervoltage
        d["PF_3Phase_means"] = get_pf1_means(value_list)
        d["PF_3Phase_maxs"] = get_pf1_max(value_list)
        d["PF_3Phase_mins"] = get_pf1_min(value_list)
        d["PF_3Phase_medians"] = get_pf1_median(value_list)
        d["active_powers"] = calc_active_power(value_list,k_factor)

    print('\n____________________________ if mins_info === 10______________ \n')
            
    u_10  =  json["subassemblies"][subassembly_instance]["collectors"]["pf1"]["CSV_10"][0]
    
    if "CSV_20" in json["subassemblies"][subassembly_instance]["collectors"]["pf1"]:
        u_20  =  json["subassemblies"][subassembly_instance]["collectors"]["pf1"]["CSV_20"][0]
    else: 
        u_20 = []

    '''
    WHEN U_6 IS NOT NULL____________________________________________________________________________________________________
    '''
    
    value_list_6 = get_sensor_data_values(json["subassemblies"][subassembly_instance],"CSV_6")
    if value_list_6.size > 0:     
        d["voltage_thd_means"] = get_voltage_thd_means(value_list_6)
        d["current_thd_means"] = get_current_thd_means(value_list_6)
        d["voltage_thd_medians"] = get_voltage_thd_medians(value_list_6)
        d["current_thd_medians"] = get_current_thd_medians(value_list_6)
        
        d["sag_swell_means"] = get_sag_swell_means(value_list_6)
        d["sag_swell_mag_max"] = get_sag_swell_maxes(value_list_6)
        d["sag_swell_medians"] = get_sag_swell_medians(value_list_6)
        d["sag_swell_mag_min"] = get_sag_swell_mins(value_list_6)
        d["sag_counts"] = get_sag_counts(value_list_6)
        d["swell_counts"] = get_swell_counts(value_list_6)
    
    #print(Sag_Swell_3)
    '''
    WHEN U_10 IS NOT NULL_____________________________________________________________________________________________________
    '''

    value_list_10    = [[float(x) for x in f[1:]] for f in u_10]
    if value_list_10:
        d["harmonic_norm"] = get_harmonic_norm(value_list_10,d["PF_3Phase_medians"])
        d["fund_freq_unfiltered"] = int(np.mean(np.array(value_list_10)[:,12]))        
        freq = np.array(value_list_10)[:,12]
        d["fund_freq_array"] = np.array(list(filter(lambda x: x>10 , freq)))
        
        d["fund_freq"] = int(np.mean(d["fund_freq_array"]) ) if len(d["fund_freq_array"]) != 0 else 0#np.mean(fund_freq_array)
        print('\n______ printing fund freq_________ \n')
        #print(fund_freq)

        if d["fund_freq"] in range(45, 55):     # required for the calculation of over ovarvoltage, under voltage, sag , swell.
            d["voltage_overruns"] = get_voltage_overruns(value_list,264)
            d["voltage_underruns"] = get_voltage_underruns(value_list,216)
            
        else:
            d["voltage_overruns"] = get_voltage_overruns(value_list,308)
            d["voltage_underruns"] = get_voltage_underruns(value_list,252)

    '''
    WHEN U_20 IS N0T NULL_____________________________________________________________________________________________________
    '''
    if u_20 !=[]:
        time_list_20     = [[x for x in f[0]] for f in u_20]
        value_list_20    = np.array([[k_factor*float(x) for x in f[1:]] for f in u_20])

        d["active_energy_total"]    = value_list_20[:,0][len(value_list_20)-1] - value_list_20[:,0][0]
        d["apparent_energy_total"]  = value_list_20[:,1][len(value_list_20)-1] - value_list_20[:,1][0]

        #print(apparent_energy_total)
    else:
        d["active_energy_total"]=None
        d["apparent_energy_total"]=None
    return d

def transformTo10minGaugeDict(dd):
    return {
        "i_3phase": dd.get("current_means"),
        "v_3phase" : dd.get("voltage_means"),
        "ithd": dd.get("current_thd_medians"),
        "vthd": dd.get("voltage_thd_medians"),
        "batch_voltage_min": dd.get("batch_voltage_min"), 
        "voltage_underrun": dd.get("voltage_underruns"),
        "batch_voltage_max":dd.get("batch_voltage_max"), 
        "voltage_overrun":dd.get("voltage_overruns"),
        "sag_count":dd.get("sag_counts"),
        "sag_swell_mag_min": dd.get("sag_swell_mag_min"),
        "swell_count":dd.get("swell_counts"),
        "sag_swell_mag_max":dd.get("sag_swell_mag_max"),
        "fund_freq":dd.get("fund_freq"), 
        "freq_var_val":dd.get("fund_freq_array"),
        "mr_time_percentage": dd.get("Machine_running_percent_i"),
        "percent_machine_running": dd.get("Machine_running_percent_pf"),
        "pf_mean": dd.get("PF_3Phase_means"),
        "pf_median_list": dd.get("harmonic_norm"),
        "av": dd.get("av"),
    }

def make_vmart_pf(dd,phase):
    return {'MEAN':dd["PF_3Phase_means"][phase], 'MAX':dd["PF_3Phase_maxs"][phase] , 'MIN': dd["PF_3Phase_mins"][phase]}

def make_vmart_voltage_thd(dd,phase):
    return {'MEAN': dd["voltage_thd_means"][phase], 'MEDIAN': dd["voltage_thd_medians"][phase], 'CREST': 0, 'RMS': 0}

def make_vmart_current_thd(dd,phase):
    return {'MEAN': dd["current_thd_means"][phase], 'MEDIAN': dd["current_thd_medians"][phase], 'CREST': 0, 'RMS': 0}

def make_vmart_sag_swell(dd,phase):
    return {
                    'MEAN':dd["sag_swell_means"][phase], 
                    'MEDIAN':dd["sag_swell_medians"][phase] , 
                    'MAX':dd["sag_swell_mag_max"][phase] , 
                    'MIN':dd["sag_swell_mag_min"][phase] , 
                    'sag count':dd["sag_counts"][phase] , 
                    'swell count':dd["swell_counts"][phase] , 
                    'CREST': 0, 
                    'RMS': 0
                }

def make_vmart_data(dd):
    pf1=make_vmart_pf(dd,0)
    pf2=make_vmart_pf(dd,1)
    pf3=make_vmart_pf(dd,2)
    v1_thd = make_vmart_voltage_thd(dd,0)
    v2_thd = make_vmart_voltage_thd(dd,1)
    v3_thd = make_vmart_voltage_thd(dd,2)

    i1_thd = make_vmart_current_thd(dd,0)
    i2_thd = make_vmart_current_thd(dd,1)
    i3_thd = make_vmart_current_thd(dd,2)

    Sag_Swell_1 = make_vmart_sag_swell(dd,0)
    Sag_Swell_2 = make_vmart_sag_swell(dd,1)
    Sag_Swell_3 = make_vmart_sag_swell(dd,2)

    pf1 = {
        'Phase-1 Voltage': {'MEAN': dd["voltage_means"][0]}, 
        'Phase-1 Current': {'MEAN': dd["current_means"][0]}, 
        'Phase-1 PF': pf1, 
        'Phase-1 Active Power': {'MEAN':dd["active_powers"][0]},
        'Phase-1 Voltage Harmonic Distortion': v1_thd, 
        'Phase-1 Current Harmonic Distortion': i1_thd, 
        'Phase-1 Sag and Swell': Sag_Swell_1,

        'Phase-2 Voltage': {'MEAN': dd["voltage_means"][1]},
        'Phase-2 Current': {'MEAN': dd["current_means"][1]}, 
        'Phase-2 PF': pf2, 
        'Phase-2 Active Power': {'MEAN': dd["active_powers"][1]}, 
        'Phase-2 Voltage Harmonic Distortion': v2_thd, 
        'Phase-2 Current Harmonic Distortion': i2_thd,
        'Phase-2 Sag and Swell': Sag_Swell_2,
                                                                            
        'Phase-3 Voltage': {'MEAN': dd["voltage_means"][2]}, 
        'Phase-3 Current': {'MEAN': dd["current_means"][2]}, 
        'Phase-3 PF': pf3, 
        'Phase-3 Active Power': {'MEAN': dd["active_powers"][2]}, 
        'Phase-3 Voltage Harmonic Distortion': v3_thd, 
        'Phase-3 Current Harmonic Distortion': i3_thd, 
        'Phase-3 Sag and Swell': Sag_Swell_3 ,
                                                                            
        'Active Energy': {'total':dd["active_energy_total"]}, 
        'Apparent Energy': {'total': dd["apparent_energy_total"]}, 
        'META': {'energyStats': True}, 
        'Phase-3 Active Energy': {'total': 0},
        'Phase-2 Active Energy': {'total': 0},
        'Phase-1 Active Energy': {'total': 0}, 
        'Machine_Util':{'Mach_Util_10_mins':100}, 
        'Link Stats': { 
            'Dataloss_Percentage_Type_20': 0, 
            'MTTR': 0, 'Dataloss_Percentage_Type_4': 0.83, 
            'Dataloss_Percentage_Type_6': 0, 
            'MTBF': 9.96
        }
    }
    return pf1
         
def input_processing(json, subassembly_instance, subassembly, machine_parameters, mins_info, stored_object_within_24hrs,machineid,json_final_output,include_vmart=True):
    '''
    Taking the raw data from FIRMWARE and making required data frame for processing
    '''
    gr=get_machine_rules(machineid)
    print('gr: ',gr)
    k_factor = int(gr.get('metas',{}).get('kFactor',1))
    cttype = int(gr.get('metas',{}).get('cttype',100))
    gauge_rule_list = gr.get('gauge_rule_list',{})
    print("Gauge Rule List: ",gauge_rule_list)

    try:
        if mins_info == '10':
            dd=preprocess_10min_data(json,subassembly_instance,k_factor)

            print('\n before calling 10 min gauges \n')
            min_10_dict = transformTo10minGaugeDict(dd)
            gauge_output_list    = min_10_gauges(min_10_dict,gauge_rule_list,subassembly_instance,cttype,k_factor, subassembly, stored_object_within_24hrs, machineid)

            json_final_output['gauge'].update(gauge_output_list)
            if include_vmart:
                pf1 = make_vmart_data(dd)
                vmart_subassembly_list = {
                    subassembly_instance: {
                        'collectors': {
                            'pf1':pf1
                        }
                    }
                }
                json_final_output['subassemblies'].update(vmart_subassembly_list)
            json_final_output.update(machine_parameters={'k_factor': float(k_factor)})            
            return json_final_output


        print('\n\n 999999999999999999999999999999999999999999999______before calling 1 min json files______\n')
        min_1_dict = make_min_1_dict(json,subassembly_instance,k_factor)
        gauge_output_list  = min_1_gauges(min_1_dict,subassembly_instance, subassembly )
        json_final_output['gauge'].update(gauge_output_list)
        json_final_output['subassemblies'].update({'a' : 'no output' })
        json_final_output.update(machine_parameters={'k_factor': k_factor})
        
        #print('\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~PRINTING OUTPUT~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n',output)                  

        return json_final_output
      
    except Exception as e:
        import sys
        exec_info=sys.exc_info()[2]
        line=exec_info.tb_lineno
        print('\n\n ()()()()()()()()()()()() ERROR OCCURED in INPUT PROCESSING () {}     ()()()()() at   {}'.format(str(e),line) )
        print(traceback.format_exc())
        return json_final_output
    
