
'''
                                                 10 MIN GAUGES
'''

from Additional_functions import *
from itertools import groupby
import time
from GaugeCurrentImbalance import GaugeCurrentImbalance
from GaugeVoltageImbalance import GaugeVoltageImbalance
from GaugeCurrentThd import GaugeCurrentThd
from GaugeVoltageThd import GaugeVoltageThd
from GaugeUnderVoltage import GaugeUnderVoltage
from GaugeOverVoltage import GaugeOverVoltage
from GaugeSag import GaugeSag
from GaugeSwell import GaugeSwell
from GaugeFrequencyVariation import GaugeFrequencyVariation
from GaugeMachineUtilization import GaugeMachineUtilization, GaugeUsage
from GaugePowerUtilizationUnderLoad import GaugePowerUtilizationUnderLoad
from GaugeStatorWindingFailure import GaugeStatorWindingFailure
from GaugeMotorBearingFailure import GaugeMotorBearingFailure
from GaugeHeaterFailure import GaugeHeaterFailure
from pynovatec_compute.gauges import GenGauges

import traceback
import simplejson as json
from simplenumpyencoder import NumpyEncoder


def get_default_10min_gauge_list(subassembly):
    gauge_list = [
        GaugeCurrentImbalance,
        GaugeVoltageImbalance,
        GaugeCurrentThd,
        GaugeVoltageThd,
        GaugeUnderVoltage,
        GaugeOverVoltage,
        GaugeSag,
        GaugeSwell,
        GaugeFrequencyVariation,
        GaugePowerUtilizationUnderLoad,
        GaugeStatorWindingFailure,
        GaugeMotorBearingFailure,
        GaugeHeaterFailure
    ]

    # Handle energysense different it is 'Usage' otherwise it is Machine Utiliztion.

    if subassembly.lower() =='energysense':
        gauge_list.append(GaugeUsage)
    else:
        gauge_list.append(GaugeMachineUtilization)
    return gauge_list


gauge_dict_10min = {
    'vfdinput': [
        GaugeCurrentImbalance,
        GaugeVoltageImbalance,
        GaugeCurrentThd,
        GaugeVoltageThd,
        GaugeUnderVoltage,
        GaugeOverVoltage,
        GaugeSag,
        GaugeSwell,
        GaugeFrequencyVariation,
        GaugeHeaterFailure,
        GaugeMachineUtilization
    ],
    'vfdoutput': [
        GaugeCurrentImbalance,
        GaugeVoltageImbalance,
        GaugeCurrentThd,
        GaugePowerUtilizationUnderLoad,
        GaugeStatorWindingFailure,
        GaugeMotorBearingFailure,
        GaugeHeaterFailure,
        GaugeMachineUtilization
    ]

}


def get_10min_gauge_list(subassembly):
    low_sub= subassembly.lower()
    if low_sub in gauge_dict_10min:
        return gauge_dict_10min[low_sub]
    return get_default_10min_gauge_list(subassembly)    


def  min_10_gauges(min_10_dict,gauge_rule_list,subassembly_instance,cttype, k_factor, subassembly,stored_object_within_24hrs,machineid,output_format=1):
    try:
        # Getting machine running status.
        # 1 for Machine running True, 0 for False
        # Success will be True if the machine is running 
        mr_time_percentage = min_10_dict["mr_time_percentage"]
        if mr_time_percentage is not None:
            Machine_status     = 0 if mr_time_percentage < 10 else 1
        else:
            Machine_status=0

        gauge_kwargs = {
            "subassembly": subassembly,
            "subassembly_instance": subassembly_instance,
            "cttype": cttype,
            "k_factor": k_factor,
            "gauge_rule_list": gauge_rule_list,
            "Machine_status": Machine_status,
            "stored_object": stored_object_within_24hrs,
            "machineid":  machineid,
            "current_datetime_stamp": stored_object_within_24hrs.current_ts(),
            "output_format": output_format 
        }

        gauge_kwargs.update(min_10_dict)
        gauge_list = get_10min_gauge_list(subassembly)
        return GenGauges(gauge_list,**gauge_kwargs)
        
    except Exception as e:
        import sys
        exec_info=sys.exc_info()[2]
        line=exec_info.tb_lineno
        print('\n ##############  ERROR IN MIN 10 GAUGES  in  {}  at {} ' .format(str(e),line))
        print(traceback.format_exc())

