 # -*- coding: utf-8 -*-

from input_processing import *
from Additional_functions import *
from MachineStoredObject import MachineStoredObject
from input_processing import preprocess_10min_data, transformTo10minGaugeDict, make_vmart_data
from pynovatec_compute.batchprocessing import JsonBatchProcessor

class PowerAnalyzerBatchProcessor(JsonBatchProcessor):
    def __init__(self,*args,version=1910.2021,**kwargs):
        super().__init__(*args,**kwargs)
        self.version=1910.2021

    def get_default_header(self, output_format = format, version = 1910.2021):
        json_final_output={'header' : self.input_batch_header,self.gauge_key:{},'subassemblies':{}, 'version':version}
        json_final_output['header']['output_format'] = output_format
        return json_final_output

class PowerAnalyzerOneMinBatchProcessor(PowerAnalyzerBatchProcessor):
    def process(self,input_json, output_format = 1, include_vmart=False):
        result = self.process_1_min_batch(input_json,output_format = output_format)
        return result

    def get_batch_filename(self):
        return str(self.batch_timestamp) + '.json'
    
    def process_1_min_batch(self,input_json, output_format=1):
        k_factor = int(self.gr.get('metas',{}).get('kFactor',1))
        json_final_output = self.get_default_header(output_format = output_format)
        for subassembly_instance,subassembly in input_json['subassemblies'].items():
            subassembly_name = subassembly['header']['subAssembly']['name'].lower()
            min_1_dict = make_min_1_dict(input_json,subassembly_instance,k_factor)
            gauge_output_list  = min_1_gauges(min_1_dict,subassembly_instance, subassembly_name, output_format=output_format )
            # Lines below are for raw gauge test.
            # raw_output_list  = min_1_raw_gauges(input_json,subassembly_instance, subassembly_name, output_format=output_format )
            # print("Raw gauges: ",json.dumps(raw_output_list,indent=3))
            # gauge_output_list[subassembly_instance].update(raw_output_list[subassembly_instance])
            json_final_output[self.gauge_key].update(gauge_output_list)
        json_final_output['subassemblies'].update({'a' : 'no output' })
        json_final_output.update(machine_parameters={'k_factor': k_factor})
        return json_final_output


class PowerAnalyzerTenMinBatchProcessor(PowerAnalyzerBatchProcessor):
    def process(self,input_json, output_format = 1, include_vmart=True):
        result = self.process_10_min_batch(input_json,output_format=output_format, include_vmart = include_vmart)
        return result

    def process_10_min_batch(self,input_json, output_format=1,include_vmart = True):
        k_factor = int(self.gr.get('metas',{}).get('kFactor',1))
        cttype = int(self.gr.get('metas',{}).get('cttype',100))
        stored_object = MachineStoredObject(input_json["header"],self.current_timestamp)
        json_final_output = self.get_default_header(output_format = output_format)
        for subassembly_instance,subassembly in input_json['subassemblies'].items():
            subassembly_name = subassembly['header']['subAssembly']['name'].lower()
            dd=preprocess_10min_data(input_json,subassembly_instance,k_factor)
            min_10_dict = transformTo10minGaugeDict(dd)
            gauge_output_list = min_10_gauges(min_10_dict,self.gauge_rule_list,subassembly_instance,cttype,k_factor, subassembly_name, stored_object, self.machineid, output_format=output_format) 
            json_final_output[self.gauge_key].update(gauge_output_list)
            if include_vmart:
                try:
                    pf1 = make_vmart_data(dd)
                    vmart_subassembly_list = {
                        subassembly_instance: {
                            'collectors': {
                                'pf1':pf1
                            }
                        }
                    }
                except Exception as e:
                    vmart_subassembly_list = {'a': 'Missing data'}
                json_final_output['subassemblies'].update(vmart_subassembly_list)

        stored_object.save()
        return json_final_output
