
''' This module consists of functions needed to log errors or issues in any code execution '''

import json
import traceback
import time

class LoggerBase:
    pass

class JsonLogger(LoggerBase):
    def __init__(self, file, keeptime=24*3600):
        self.file = file
        self.keeptime = keeptime
        self.log_json = self.open()
        self.isDirty = False
    
    def open(self):
        try:
            with open(self.file) as log:
                self.log_json=json.load(log)
        except FileNotFoundError:
            self.log_json={}
        return self.log_json

    def save(self):
        if self.isDirty:
            with open(self.file,"w") as log:    
                json.dump(self.log_json,log,indent=4)

    def log(self,current_datetime_stamp,func_name,log_type,log_text):
        log_key = str(current_datetime_stamp)
        if log_key not in self.log_json:
            self.log_json[str(current_datetime_stamp)]={}   
        if func_name not in self.log_json[log_key]:
            self.log_json[log_key][func_name]={}
        if log_type in ["error","normal"]: 
            if log_type not in self.log_json[log_key][func_name]:
                self.log_json[log_key][func_name][log_type]=[]
            self.log_json[log_key][func_name][log_type].append(log_text)
        else:
            print("log type should be 'normal' or 'error', not '{}'".format(log_type))

        self.isDirty = True

    def purge(self,current_datetime_stamp=None):
        if self.log_json=={}:
            return
        if current_datetime_stamp == None:
            current_datetime_stamp = int(time.time())
        key=sorted(list(self.log_json.keys()))[0]
        for key in list(self.log_json.keys()):
            print("Duration: ", current_datetime_stamp-int(key))
            if current_datetime_stamp-int(float(key))>self.keeptime:
                print("deleting: ",key)
                del self.log_json[key]    
                self.isDirty = True


json_logger = None

def LoggerInit(file,**kwargs):
    global json_logger

    json_logger = JsonLogger(file,**kwargs)               
        
def add_log_info(*args):
    global json_logger

    if not json_logger:
        return
    json_logger.log(*args)

def log_purge(*args):
    global json_logger

    if not json_logger:
        return
    json_logger.purge(*args)

def log_save():
    global json_logger

    if not json_logger:
        return
    json_logger.save()

